Package: monaLisa
Type: Package
Title: Binned Motif Enrichment Analysis and Visualization
Version: 1.19.0
Authors@R: c(
    person("Dania", "Machlab", email = "dania.machlab@gmail.com", role = c("aut"), 
           comment = c(ORCID = "0000-0002-2578-6930")),
    person("Lukas", "Burger", email = "lukas.burger@fmi.ch", role = c("aut"),
           comment = c(ORCID = "0000-0001-5596-7567")),
    person("Charlotte", "Soneson", email = "charlottesoneson@gmail.com", 
           role = c("aut"),
           comment = c(ORCID = "0000-0003-3833-2169")),
    person("Dany", "Mukesha", role = c("ctb"),
           comment = c(ORCID = "0009-0001-9514-751X")),
    person("Michael", "Stadler", email = "michael.stadler@fmi.ch", 
           role = c("aut", "cre"), comment = c(ORCID = "0000-0002-2269-4934")))
Description: Useful functions to work with sequence motifs in the analysis of 
    genomics data. These include methods to annotate genomic regions or 
    sequences with predicted motif hits and to identify motifs that drive 
    observed changes in accessibility or expression. Functions to produce 
    informative visualizations of the obtained results are also provided. 
Depends: R (>= 4.1)
Imports: BiocGenerics, BiocParallel, Biostrings, BSgenome, circlize,
        ComplexHeatmap (>= 2.11.1), Seqinfo, GenomicRanges, cli,
        ggplot2 (>= 4.0.0), glmnet, grDevices, grid, IRanges, methods,
        rlang, RSQLite, stabs, stats, SummarizedExperiment, S4Vectors,
        TFBSTools, tidyr, tools, utils, XVector
Suggests: BiocManager, BiocStyle, BSgenome.Mmusculus.UCSC.mm10,
        ggrepel, gridExtra, JASPAR2020, JASPAR2024, knitr, rmarkdown,
        testthat, TxDb.Mmusculus.UCSC.mm10.knownGene
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
biocViews: MotifAnnotation, Visualization, FeatureExtraction,
        Epigenetics
Config/testthat/edition: 3
URL: https://github.com/fmicompbio/monaLisa,
        https://bioconductor.org/packages/monaLisa/,
        https://fmicompbio.github.io/monaLisa/
BugReports: https://github.com/fmicompbio/monaLisa/issues
git_url: https://git.bioconductor.org/packages/monaLisa
git_branch: devel
git_last_commit: dfdf4c3
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:55:53 UTC; biocbuild
Author: Dania Machlab [aut] (ORCID: <https://orcid.org/0000-0002-2578-6930>),
  Lukas Burger [aut] (ORCID: <https://orcid.org/0000-0001-5596-7567>),
  Charlotte Soneson [aut] (ORCID:
    <https://orcid.org/0000-0003-3833-2169>),
  Dany Mukesha [ctb] (ORCID: <https://orcid.org/0009-0001-9514-751X>),
  Michael Stadler [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2269-4934>)
Maintainer: Michael Stadler <michael.stadler@fmi.ch>
